
import requests
import json
import os
import logging
from enum import Enum
from urllib.parse import quote

log = logging.getLogger(__name__)

LOGLEVEL = os.environ.get('LOGLEVEL', 'WARNING').upper()
logging.basicConfig(level=LOGLEVEL)


class Status(Enum):
    FINISHED = 'finished'
    STARTED = 'started'
    CORRECTED = 'corrected'


class Client():

    def __init__(self, apikey):
        self.fahrzeugschein = ""
        self.status = "started"
        self.session = ""
        self.endpoint = os.environ.get("REGISTRATION_RECOGNITION_ENDPOINT",
                                       "https://api.mmmint.ai/fahrzeugschein")
        self.api_version = os.environ.get("REGISTRATION_RECOGNITION_VERSION", "v1")
        self.base_url = "{}/{}".format(self.endpoint, self.api_version)
        if not apikey:
            raise UnauthorizedException()
        self.api_key = apikey

    def new_fahrzeugschein(self, fahrzeugschein_path):
        self.fahrzeugschein_path = fahrzeugschein_path

        url = '{}/fahrzeugschein?access_token={}'.format(self.base_url, self.api_key)
        log.info(url)

        with open(self.fahrzeugschein_path, 'rb') as f:
            r = requests.post(url, files={'file': f})
        return self._handle_response(r)

    def new_fahrzeugschein_url(self, fahrzeugschein_url):
        self.fahrzeugschein_url = quote(fahrzeugschein_url, safe='')
        url = '{}/fahrzeugschein/url?image_url={}&access_token={}'.format(
            self.base_url, self.fahrzeugschein_url, self.api_key)
        log.info(url)
        r = requests.post(url)
        return self._handle_response(r)

    def get_fahrzeugschein_status(self, session_id=''):
        session_id = self._handle_empty_session(session_id)
        url = '{}/fahrzeugschein/status/{}?access_token={}'.format(
            self.base_url, session_id, self.api_key)
        log.info(url)
        r = requests.get(url)
        if r.status_code == 200:
            response_dict = json.loads(r.text)
            log.info(response_dict)
            self.status = response_dict["status"]
            return self.status
        elif r.status_code != 200:
            self._handle_not_authenticated(r)
            self._handle_other(r)

    def get_fahrzeugschein(self):
        url = '{}/fahrzeugschein/{}?access_token={}'.format(
            self.base_url, self.session, self.api_key)
        log.info(url)
        r = requests.get(url)
        if r.status_code == 200:
            response_dict = json.loads(r.text)
            log.info(response_dict)
            self.fahrzeugschein = response_dict
            return self.fahrzeugschein
        elif r.status_code != 200:
            self._handle_not_authenticated(r)
            self._handle_other(r)

    def edit_fahrzeugschein(self):
        data = json.dumps(self.fahrzeugschein)
        log.info(data)

        url = '{}/fahrzeugschein/{}?access_token={}'.format(
            self.base_url, self.session, self.api_key)
        log.info(url)
        r = requests.put(url, data=data)

        if r.status_code == 200:
            response_dict = json.loads(r.text)
            log.info(response_dict)
            self.fahrzeugschein = response_dict
            return self.fahrzeugschein
        elif r.status_code != 200:
            self._handle_not_authenticated(r)
            self._handle_other(r)

    def get_sessions(self):
        url = '{}/session?access_token={}'.format(
            self.base_url, self.api_key)
        log.info(url)
        r = requests.get(url)
        if r.status_code == 200:
            response_dict = json.loads(r.text)
            log.info(response_dict)
            self.sessions = response_dict
            return self.sessions
        elif r.status_code != 200:
            self._handle_not_authenticated(r)
            self._handle_other(r)

    def is_finished(self, session_id):
        session_id = self._handle_empty_session(session_id)
        self.get_fahrzeugschein_status(session_id)

    def get_sessions_finished(self):
        sessions = self.get_sessions()
        finished_sessions = []
        for sess in sessions:
            if Status.FINISHED.value == sess["status"]:
                finished_sessions.append(sess)
        return finished_sessions

    def get_detection_bounding_boxes(self, session_id=""):
        session_id = self._handle_empty_session(session_id)
        url = '{}/detection/boundingboxes/{}?access_token={}'.format(
            self.base_url, session_id, self.api_key)
        log.info(url)
        r = requests.get(url)
        if r.status_code == 200:
            response_dict = json.loads(r.text)
            log.info(response_dict)
            self.boundingboxes = response_dict
            return self.boundingboxes
        elif r.status_code != 200:
            self._handle_not_authenticated(r)
            self._handle_other(r)

    def get_detection_image(self, session_id=''):
        session_id = self._handle_empty_session(session_id)
        url = '{}/detection/image/{}?access_token={}'.format(
            self.base_url, session_id, self.api_key)
        log.info(url)
        r = requests.get(url)
        if r.status_code == 200:
            response_dict = json.loads(r.text)
            log.info(response_dict)
            self.image = response_dict
            return self.image
        elif r.status_code != 200:
            self._handle_not_authenticated(r)
            self._handle_other(r)

    def get_detection_cropped_image(self, session_id=''):

        session_id = self._handle_empty_session(session_id)

        url = '{}/detection/croppedimage/{}?access_token={}'.format(
            self.base_url, session_id, self.api_key)
        log.info(url)
        r = requests.get(url)
        if r.status_code == 200:
            response_dict = json.loads(r.text)
            log.info(response_dict)
            self.cropped_images = response_dict
            return self.cropped_images
        elif r.status_code != 200:
            self._handle_not_authenticated(r)
            self._handle_other(r)

    def _handle_response(self, r):
        if r.status_code == 200:
            response_dict = json.loads(r.text)
            log.info(response_dict)

            self.status = response_dict["status"]
            self.session = response_dict["session_id"]
            return self.status, self.session
        elif r.status_code != 200:
            self._handle_not_authenticated(r)
            self._handle_other(r)

    def _handle_not_authenticated(self, r):
        if r.status_code == 403:
            log.error("Not authenticated")
            raise UnauthorizedException(self.api_key, r)

    def _handle_other(self, r):
        log.warning(r.reason)

    def _handle_empty_session(self, session_id):
        if not session_id:
            session_id = self.session

        if not session_id:
            raise NoSessionException()

        return session_id


class NoSessionException(Exception):
    def __init__(self, message="No session: Please provide a session_id"):
        super().__init__(self.message)


class UnauthorizedException(Exception):
    def __init__(self, api_key="", request="", message="Unauthorized: Please provide API_KEY"):
        self.api_key = api_key
        self.request = request
        self.message = message
        super().__init__(self.message)



import os
import time

from sdk import Client, Status


def main():
    apikey = os.environ.get("API_KEY", "")

    # Create an Endpoint with your Credentials
    client = Client(apikey)

    # Post Fahrzeugschein from local Image
    #image_path = "4.jpg"
    # client.post_image(image_path)
    # print(client.session)

    # Post Fahrzeugschein from URL
    image_url = "https://deinautoguide.de/wp-content/uploads/2020/06/Fahrzeugschein-Aventador.jpg"
    # client.new_fahrzeugschein_url(image_url)
    client.session = '28ea1770c52c9c5d508b7c4e3c715fae739c97d4dfa615ff847822aebf1804d2'
    print(client.session)

    # Get Status of Calculation
    while client.status == Status.STARTED.value:
        client.get_fahrzeugschein_status()
        time.sleep(2)

    # Get Results of Fahrzeugschein
    if client.status == Status.FINISHED.value:
        client.get_fahrzeugschein()

        for i in client.fahrzeugschein:
            print(i, ": ", client.fahrzeugschein[i])

        bb = client.get_detection_bounding_boxes()
        di = client.get_detection_image()
        ci = client.get_detection_cropped_image()
        print(bb)
        print(di)
        print(ci)

    # TODO: client.edit_fahrzeugschein()


if __name__ == "__main__":
    # execute only if run as a script
    main()



import os
from mmmint.registration_recognition.sdk import Status, Client
import time
import pytest


class Ressources():
    def __init__(self):
        self.APIKEY = os.environ.get("APIKEY", "")
        self.test_image_path = "sdk/mmmint/test_fahrzeugschein.jpeg"
        self.session_id = "2a9efbec7d93b47e99165f4401d58cfa840d417c7e47074b49a4d2f4d61487f5"
        self.session_created = "2020-12-16 13:44:03.666717"
        self.session_info = ""
        self.session_result = {
            "vorname": "",
            "nachname": "KREIS HETTMANN",
            "strasse": "DUSSELDORFER STRABE 26",
            "stadt": "40822 METTMANN",
            "kennzeichen": "ME-KV-1200",
            "identifizierungsnummer": "",
            "hu_jahr": "",
            "hu_monat": "",
            "erstzulassung_tag": "28",
            "erstzulassung_monat": "01",
            "erstzulassung_jahr": "2016",
            "codehersteller": "0009",
            "codetyp": ""
        }
        self.session_id_corrected = "0e5ec912c19cf8108733c51b2e46164bd297487b4908bd1c103f80720c9115d6"


def test_client_get_sessions_integration():
    ressources = Ressources()

    client = Client(ressources.APIKEY)
    sessions = client.get_sessions()

    for session in sessions:
        if session["session_id"] == ressources.session_id:
            test_session = session

    # found session?
    assert test_session["session_id"] == ressources.session_id
    # found status and is finished?
    assert test_session["status"] == Status.FINISHED.value
    # found creation date?
    assert test_session["created"] == ressources.session_created
    # found info?
    assert test_session["info"] == ressources.session_info

def test_client_get_fahrzeugschein_integration():
    ressources = Ressources()

    client = Client(ressources.APIKEY)

    client.session = ressources.session_id
    clientResult = client.get_fahrzeugschein()

    for key in ressources.session_result.keys():
        assert ressources.session_result[key] == clientResult[key]

def test_new_fahrzeugschein_integration():
    ressources = Ressources()

    # Session started sucessfully?
    client = Client(ressources.APIKEY)
    client.new_fahrzeugschein(ressources.test_image_path)
    assert client.status == Status.STARTED.value

    #Get resulsts or Timeout?
    resultTimeout = Timeout()
    while client.status == Status.STARTED.value and resultTimeout.timeout == False:
        client.get_fahrzeugschein_status()
        resultTimeout.update()
    assert resultTimeout.timeout == False


    #found Fahrzeugschein?
    client.get_fahrzeugschein()
    assert client.status == Status.FINISHED.value
    for key in ressources.session_result.keys():
        if key not in client.fahrzeugschein.keys():
            assert client.fahrzeugschein[key]

class Timeout():

    def __init__(self):
        self.limit = 30
        self.t0 = time.process_time()
        self.t1 = 0
        self.elapsedTime = 0
        self.timeout = False
    
    def update(self):
        time.sleep(2)
        self.t1 = self.t1 + time.process_time()
        self.elapsedTime = self.t1 - self.t0
        if self.elapsedTime > self.limit:
            self.timeout = True

    



from setuptools import setup, find_packages

import os
import re

WORKFLOW_RELEASE = "Release"
NAME_PROD = "mmmint_registration_recognition"
NAME_DEV = "mmmint_registration_recognition"


def getVariables():

    workflow_name = os.environ.get("GITHUB_WORKFLOW", "")

    if WORKFLOW_RELEASE == workflow_name:
        version = getVersionProd()
        name = NAME_PROD
    else:
        version = getVersionDev()
        name = NAME_DEV

    return name, version


def getVersionProd():
    REF_PREFIX = "refs/tags/"

    github_ref = os.environ.get("GITHUB_REF", 'refs/tags/0')
    return re.sub(REF_PREFIX, '', github_ref)


def getVersionDev():
    return os.environ.get("GITHUB_RUN_NUMBER", "0")


with open("sdk/mmmint/README.md", "r") as fh:
    long_description = fh.read()

with open('sdk/mmmint/requirements.txt') as f:
    required = f.read().splitlines()

NAME, VERSION = getVariables()

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Intended Audience :: System Administrators',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8'
]

setup(
    name=NAME,
    version=VERSION,
    author="MMM Intelligence UG",
    author_email="info@mmmint.ai.com",
    classifiers=CLASSIFIERS,
    description="An interface to execute Fahrzeugschein API commands using Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://api.mmmint.ai/fahrzeugschein/v1/docs",
    package_dir={'': 'sdk'},
    packages=find_packages(where='sdk'),
    include_package_data=True,
    python_requires='>=3.7',
    install_requires=required
)
